# API Docs - v1.0.3

!!! Info "Tested Siddhi Core version: *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/">5.1.5</a>*"
    It could also support other Siddhi Core minor versions.

## Sinkmapper

### protobuf *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/#sink-mapper">(Sink Mapper)</a>*
<p></p>
<p style="word-wrap: break-word;margin: 0;">This output mapper allows you to convert Events to protobuf messages before publishing them. To work with this mapper you have to add auto-generated protobuf classes to the project classpath. When you use this output mapper, you can either define stream attributes as the same names as the protobuf message attributes or you can use custom mapping to map stream definition attributes with the protobuf attributes.Please find the sample proto definition [here](https://github.com/siddhi-io/siddhi-map-protobuf/tree/master/component/src/main/resources/sample.proto) </p>
<p></p>
<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>

```
@sink(..., @map(type="protobuf", class="<STRING>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">class</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This specifies the class name of the protobuf message class, If sink type is grpc then it's not necessary to provide this parameter.</p></td>
        <td style="vertical-align: top">-</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@sink(type='grpc',  url = 'grpc://localhost:2000/org.wso2.grpc.test.MyService/process 
@map(type='protobuf')) 
define stream BarStream (stringValue string, intValue int,longValue long,booleanValue bool,floatValue float,doubleValue double)
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above definition will map BarStream values into the protobuf message type of the 'process' method in 'MyService' service</p>
<p></p>
<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@sink(type='grpc', url = 'grpc://localhost:2000/org.wso2.grpc.test.MyService/process
@map(type='protobuf'), 
@payload(stringValue='a',longValue='b',intValue='c',booleanValue='d',floatValue = 'e', doubleValue  = 'f'))) 
define stream BarStream (a string, b long, c int,d bool,e float,f double);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">The above definition will map BarStream values to request message type of the 'process' method in 'MyService' service. and stream values will map like this, <br>- value of 'a' will be assign 'stringValue' variable in the message class <br>- value of 'b' will be assign 'longValue' variable in the message class <br>- value of 'c' will be assign 'intValue' variable in the message class <br>- value of 'd' will be assign 'booleanValue' variable in the message class <br>- value of 'e' will be assign 'floatValue' variable in the message class <br>- value of 'f' will be assign 'doubleValue' variable in the message class <br></p>
<p></p>
<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
@sink(type='grpc', url = 'grpc://localhost:2000/org.wso2.grpc.test.MyService/testMap' 
@map(type='protobuf')) 
 define stream BarStream (stringValue string,intValue int,map object);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">The above definition will map BarStream values to request message type of the 'testMap' method in 'MyService' service and since there is an object data type is inthe stream(map object) , mapper will assume that 'map' is an instance of  'java.util.Map' class, otherwise it will throws and error. <br></p>
<p></p>
<span id="example-4" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 4</span>
```
@sink(type='inMemory', topic='test01', 
@map(type='protobuf', class='org.wso2.grpc.test.Request'))
define stream BarStream (stringValue string, intValue int,longValue long,booleanValue bool,floatValue float,doubleValue double);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">The above definition will map BarStream values to 'org.wso2.grpc.test.Request'protobuf class type. If sink type is not a grpc, sink is expecting to get the mapping protobuf class from the 'class' parameter in the @map extension</p>
<p></p>
## Sourcemapper

### protobuf *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/#source-mapper">(Source Mapper)</a>*
<p></p>
<p style="word-wrap: break-word;margin: 0;">This input mapper allows you to convert protobuf messages into Events. To work with this input mapper you have to add auto-generated protobuf classes to the project classpath. When you use this input mapper, you can either define stream attributes as the same names as the protobuf message attributes or you can use custom mapping to map stream definition attributes with the protobuf attributes.Please find the sample proto definition [here](https://github.com/siddhi-io/siddhi-map-protobuf/tree/master/component/src/main/resources/sample.proto) </p>
<p></p>
<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>

```
@source(..., @map(type="protobuf", class="<STRING>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">class</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This specifies the class name of the protobuf message class, If sink type is grpc then it's not necessary to provide this field.</p></td>
        <td style="vertical-align: top">-</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
source(type='grpc', receiver.url = 'grpc://localhost: 2000/org.wso2.grpc.test.MyService/process', 
@map(type='protobuf')) define stream FooStream (stringValue string, intValue int,longValue long,booleanValue bool,floatValue float,doubleValue double); 
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above definition will convert the protobuf messages that are received to this source into siddhi events.</p>
<p></p>
<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
source(type='grpc', receiver.url = 'grpc://localhost: 2000/org.wso2.grpc.test.MyService/process', 
@map(type='protobuf', @attributes(a = 'stringValue', b = 'intValue', c = 'longValue',d = 'booleanValue',' e = floatValue', f ='doubleValue'))) 
define stream FooStream (a string ,c long,b int, d bool,e float,f double);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above definition will convert the protobuf messages that are received to this source into siddhi events. since there's a mapping available for the stream, protobuf message object will be map like this, <br>-'stringValue' of the protobuf message will be assign to the 'a' attribute of the stream <br>- 'intValue' of the protobuf message will be assign to the 'b' attribute of the stream <br>- 'longValue' of the protobuf message will be assign to the 'c' attribute of the stream <br>- 'booleanValue' of the protobuf message will be assign to the 'd' attribute of the stream <br>- 'floatValue' of the protobuf message will be assign to the 'e' attribute of the stream <br>- 'doubleValue' of the protobuf message will be assign to the 'f' attribute of the stream <br></p>
<p></p>
<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
source(type='grpc', receiver.url = 'grpc://localhost: 2000/org.wso2.grpc.test.MyService/testMap', 
@map(type='protobuf')) 
define stream FooStream (stringValue string ,intValue int,map object);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above definition will convert the protobuf messages that are received to this source into siddhi events. since there's an object type attribute available in the stream (map object), mapper will assume that object is an instance of 'java.util.Map' class. otherwise mapper will throws an exception</p>
<p></p>
<span id="example-4" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 4</span>
```
@source(type='inMemory', topic='test01', 
@map(type='protobuf', class='org.wso2.grpc.test.Request')) 
define stream FooStream (stringValue string, intValue int,longValue long,booleanValue bool,floatValue float,doubleValue double); 

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">The above definition will convert the 'org.wso2.grpc.test.Request' type protobuf messages into siddhi events. If we did not provide the 'receiver.url' in the stream definition we have to provide the protobuf class name in the 'class' parameter inside @map.</p>
<p></p>
